# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{fujita2021empirical,
  title={An Empirical Investigation of Online News Classification on an Open-Domain, Large-Scale and High-Quality Dataset in Vietnamese},
  author={Fujita, H and Perez-Meana, H},
  booktitle={New Trends in Intelligent Software Methodologies, Tools and Techniques: Proceedings of the 20th International Conference on New Trends in Intelligent Software Methodologies, Tools and Techniques (SoMeT_21)},
  volume={337},
  pages={367},
  year={2021},
  organization={IOS Press}
}
"""

_DATASETNAME = "uit_vion"


_DESCRIPTION = """\
UIT-ViON (Vietnamese Online Newspaper) is a dataset collected from well-known online newspapers in Vietnamese.
The UIT-ViON is an open-domain, large-scale, and high-quality dataset consisting of 260,000 textual data
points annotated with 13 different categories for evaluating Vietnamese short text classification.
The dataset is split into training, validation, and test sets, each containing 208000, 26000,
and 26000 pieces of text, respectively.
"""

_HOMEPAGE = "https://github.com/kh4nh12/UIT-ViON-Dataset"

_LANGUAGES = ["vie"]

_LICENSE = Licenses.UNKNOWN.value

_LOCAL = False

_URLS = {
    _DATASETNAME: "https://github.com/kh4nh12/UIT-ViON-Dataset/archive/refs/heads/master.zip",
}

_SUPPORTED_TASKS = [Tasks.INTENT_CLASSIFICATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class UitVion(datasets.GeneratorBasedBuilder):
    """UIT-ViON (Vietnamese Online Newspaper) is a dataset collected from well-known online newspapers in Vietnamese."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    LABEL_CLASSES = [i for i in range(13)]

    SEACROWD_SCHEMA_NAME = "text"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "title": datasets.Value("string"),
                    "link": datasets.Value("string"),
                    "label": datasets.ClassLabel(names=self.LABEL_CLASSES),
                }
            )

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text_features(self.LABEL_CLASSES)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)
        file_dir = os.path.join("UIT-ViON-Dataset-main", "data.zip")
        data_dir = os.path.join(data_dir, file_dir)
        data_dir = dl_manager.download_and_extract(data_dir)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "UIT-ViON_train.csv"),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "UIT-ViON_test.csv"),
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "UIT-ViON_dev.csv"),
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        data = pd.read_csv(filepath)

        if self.config.schema == "source":
            for i, row in data.iterrows():
                yield i, {
                    "title": str(row["title"]),
                    "link": str(row["link"]),
                    "label": row["label"],
                }

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            for i, row in data.iterrows():
                yield i, {
                    "id": str(i),
                    "text": str(row["title"]),
                    "label": int(row["label"]),
                }



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from posixpath import split
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import (DEFAULT_SEACROWD_VIEW_NAME,
                                       DEFAULT_SOURCE_VIEW_NAME, Tasks)
import glob

_DATASETNAME = "indo4b"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_CITATION = """\
    @inproceedings{wilie-etal-2020-indonlu,
        title = "{I}ndo{NLU}: Benchmark and Resources for Evaluating {I}ndonesian 
            Natural Language Understanding",
        author = "Wilie, Bryan  and
          Vincentio, Karissa  and
          Winata, Genta Indra  and
          Cahyawijaya, Samuel  and
          Li, Xiaohong  and
          Lim, Zhi Yuan  and
          Soleman, Sidik  and
          Mahendra, Rahmad  and
          Fung, Pascale  and
          Bahar, Syafri  and
          Purwarianti, Ayu",
        booktitle = "Proceedings of the 1st Conference of the Asia-Pacific Chapter of the 
                Association for Computational Linguistics and the 10th International Joint 
                Conference on Natural Language Processing",
        month = dec,
        year = "2020",
        address = "Suzhou, China",
        publisher = "Association for Computational Linguistics",
        url = "https://aclanthology.org/2020.aacl-main.85",
        pages = "843--857",
        abstract = "Although Indonesian is known to be the fourth most frequently used language 
            over the internet, the research progress on this language in natural language processing (NLP) 
            is slow-moving due to a lack of available resources. In response, we introduce the first-ever vast 
            resource for training, evaluation, and benchmarking on Indonesian natural language understanding 
            (IndoNLU) tasks. IndoNLU includes twelve tasks, ranging from single sentence classification to 
            pair-sentences sequence labeling with different levels of complexity. The datasets for the tasks 
            lie in different domains and styles to ensure task diversity. We also provide a set of Indonesian 
            pre-trained models (IndoBERT) trained from a large and clean Indonesian dataset (Indo4B) collected 
            from publicly available sources such as social media texts, blogs, news, and websites. 
            We release baseline models for all twelve tasks, as well as the framework for benchmark evaluation, 
            thus enabling everyone to benchmark their system performances.",
    }
"""

_DESCRIPTION = """\
    Indo4B is a large-scale Indonesian self-supervised pre-training corpus
    consists of around 3.6B words, with around 250M sentences. The corpus
    covers both formal and colloquial Indonesian sentences compiled from 
    12 sources, of which two cover Indonesian colloquial language, eight
    cover formal Indonesian language, and the rest have a mixed style of
    both colloquial and formal.
"""

_HOMEPAGE = "https://github.com/IndoNLP/indonlu"

_LICENSE = "CC0"

_LANGUAGES_MAP = {
    "ind": "id",
    "jav": "jv",
    "sun": "su",
}

_URLS = {
    "indo4b": "https://storage.googleapis.com/babert-pretraining/IndoNLU_finals/dataset/preprocessed/dataset_wot_uncased_blanklines.tar.xz",
}

_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

class Indo4B(datasets.GeneratorBasedBuilder):
    """Indo4B is a large-scale Indonesian self-supervised pre-training corpus
    consists of around 3.6B words, with around 250M sentences."""

    DEFAULT_CONFIG_NAME = "indo4b_source"

    
    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="indo4b_source",
            version=_SOURCE_VERSION,
            description="Indo4B source schema",
            schema="source",
            subset_id="indo4b",
        ),
        SEACrowdConfig(
            name="indo4b_seacrowd_ssp",
            version=_SEACROWD_VERSION,
            description="Indo4B Nusantara schema",
            schema="seacrowd_ssp",
            subset_id="indo4b",
        ),
    ]

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_ssp":
            features = schemas.self_supervised_pretraining.features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        url = _URLS["indo4b"]
        path = dl_manager.download_and_extract(url) + "/processed_uncased_blanklines"

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": path,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        counter = 0
        for txt_path in glob.glob(f'{filepath}/*.txt'):
            with open(txt_path, encoding="utf-8") as f:
                if self.config.schema == "source":
                    for row in f:
                        if row.strip() != "":
                            yield (
                                counter,
                                {
                                    "id": str(counter),
                                    "text": row.strip(),
                                },
                            )
                            counter += 1
                elif self.config.schema == "seacrowd_ssp":
                    for row in f:
                        if row.strip() != "":
                            yield (
                                counter,
                                {
                                    "id": str(counter),
                                    "text": row.strip(),
                                },
                            )
                            counter += 1


# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@article{gonzales_broadening_2023,
  author    = {Gonzales, Wilkinson Daniel Wong},
  title     = {Broadening horizons in the diachronic and sociolinguisstic study of
  Philippine Englishes with the Twitter Corpus of Philippine Englishes (TCOPE)},
  journal   = {English World-Wide},
  year      = {2023},
  url       = {https://osf.io/k3qzx},
  doi       = {10.17605/OSF.IO/3Q5PW},
}
"""

_LOCAL = False
_LANGUAGES = ["eng", "fil"]
_DATASETNAME = "tcope"
_DESCRIPTION = """
The TCOPE dataset consists of public tweets (amounting to about 13.5 million words) collected from 13 major cities from the Philippines.
Tweets are either purely in English or involve code-switching between English and Filipino.
Tweets are tagged for part-of-speech and dependency parsing using spaCy. Tweets collected are from 2010 to 2021.
The publicly available dataset is only a random sample (10%) from the whole TCOPE dataset, which consist of roughly 27 million tweets
(amounting to about 135 million words) collected from 29 major cities during the same date range.
"""

_HOMEPAGE = "https://osf.io/3q5pw/wiki/home/"
_LICENSE = Licenses.CC0_1_0.value
_URL = "https://files.osf.io/v1/resources/3q5pw/providers/osfstorage/63737a5b0e715d3616a998f7"

_SUPPORTED_TASKS = [Tasks.POS_TAGGING, Tasks.DEPENDENCY_PARSING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class TCOPEDataset(datasets.GeneratorBasedBuilder):
    """TCOPE is a dataset of Philippine English tweets by Gonzales (2023)."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    # Actual data has invalid "labels" likely due to coding errors,
    # such as "BODY", "BIRTHDAY", "HAVAIANAS", etc. Only valid
    # POS tags are included here and in loaded data.
    POS_LABELS = ["NOUN", "PUNCT", "PROPN", "VERB", "PRON", "ADP", "ADJ", "ADV", "DET", "AUX", "PART", "CCONJ", "INTJ", "SPACE", "SCONJ", "NUM", "X", "SYM"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_seq_label",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd sequence labeling schema",
            schema="seacrowd_seq_label",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = "tcope_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "copeid": datasets.Value("string"),
                    "userid": datasets.Value("int64"),
                    "divided_tweet": datasets.Value("string"),
                    "postag": datasets.Value("string"),
                    "deptag": datasets.Value("string"),
                    "citycode": datasets.Value("string"),
                    "year": datasets.Value("int64"),
                    "extendedcope": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(label_names=self.POS_LABELS)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        # First ZIP contains second ZIP
        # Second ZIP has spreadsheet data
        folder_zip_dir = dl_manager.download_and_extract(_URL)
        spreadsheet_zip_dir = dl_manager.extract(f"{folder_zip_dir}/public_v1/spreadsheet_format.zip")
        spreadsheet_fp = f"{spreadsheet_zip_dir}/spreadsheet_format/tcope_v1_public_sample.csv"

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": spreadsheet_fp,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        if self.config.schema not in ("source", "seacrowd_seq_label"):
            raise ValueError(f"Received unexpected config schema {self.config.schema}")

        df = pd.read_csv(filepath, index_col=None)
        df = df.rename(columns={"divided.tweet": "divided_tweet"}).query("divided_tweet.notna()")

        for index, row in df.iterrows():
            if self.config.schema == "source":
                example = row.to_dict()
            elif self.config.schema == "seacrowd_seq_label":
                tokens, tags = self.split_token_and_tag(row["postag"], valid_tags=self.POS_LABELS)
                example = {
                    "id": str(index),
                    "tokens": tokens,
                    "labels": tags,
                }
            yield index, example

    def split_token_and_tag(self, tweet: str, valid_tags: List[str]) -> Tuple[List[str], List[str]]:
        """Split tweet into two separate lists of tokens and tags."""
        tokens_with_tags = tweet.split()
        tokens = []
        tags = []
        for indiv_token_with_tag in tokens_with_tags:
            token, tag = indiv_token_with_tag.rsplit("_", 1)
            tokens.append(token)
            if tag in valid_tags:
                tags.append(tag)
            else:  # Use "X"/other spaCy tag for invalid POS tags
                tags.append("X")
        return tokens, tags



from pathlib import Path

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks


_CITATION = ""

_DATASETNAME = "thai_gpteacher"

_DESCRIPTION = """This is a Thai-instructed dataset translated using Google Cloud Translation from GPTeacher, a
collection of modular datasets generated by GPT-4, General-Instruct & Roleplay-Instruct and is comprised of around
20,000 examples with deduplication. The dataset was asked to include reasoning and thought steps in the example
responses where appropriate.
"""

_HOMEPAGE = "https://huggingface.co/datasets/Thaweewat/gpteacher-20k-th"

_LANGUAGES = ["tha"]

_LICENSE = Licenses.CC_BY_SA_3_0.value

_LOCAL = False

_URLS = {
    _DATASETNAME: {
        "train": {
            "qa": "https://huggingface.co/datasets/Thaweewat/gpteacher-20k-th/resolve/main/gpteacher-gpt4-instruct-qa-18k-th.parquet",
            "role_play": "https://huggingface.co/datasets/Thaweewat/gpteacher-20k-th/resolve/main/gpteacher-gpt4-instruct-roleplay-2k-th.parquet",
        }
    },
}

_SUPPORTED_TASKS = [Tasks.INSTRUCTION_TUNING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class ThaiGPTeacherDataset(datasets.GeneratorBasedBuilder):
    """Thai-instructed dataset translated using Google Cloud Translation from GPTeacher."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_t2t",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "instruction": datasets.Value("string"),
                    "input": datasets.Value("string"),
                    "output": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> list[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_dir, "split": "train"},
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> tuple[int, dict]:
        df1 = pd.read_parquet(filepath[split]["qa"])
        df2 = pd.read_parquet(filepath[split]["role_play"])
        df = pd.concat([df1, df2], ignore_index=True)
        if self.config.schema == "source":
            for i, row in df.iterrows():
                yield i, {"instruction": row["instruction"], "input": row["input"], "output": row["output"]}

        elif self.config.schema == "seacrowd_t2t":
            for i, row in df.iterrows():
                yield i, {
                    "id": str(i),
                    "text_1": row["instruction"] + "\n" + row["input"],
                    "text_2": row["output"],
                    "text_1_name": "instruction + input",
                    "text_2_name": "output",
                }



from pathlib import Path
from typing import List

import datasets
import json

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, DEFAULT_SOURCE_VIEW_NAME, DEFAULT_SEACROWD_VIEW_NAME

_DATASETNAME = "parallel_su_id"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["ind", "sun"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False
_CITATION = """\
@INPROCEEDINGS{7437678,
  author={Suryani, Arie Ardiyanti and Widyantoro, Dwi Hendratmo and Purwarianti, Ayu and Sudaryat, Yayat},
  booktitle={2015 International Conference on Information Technology Systems and Innovation (ICITSI)}, 
  title={Experiment on a phrase-based statistical machine translation using PoS Tag information for Sundanese into Indonesian}, 
  year={2015},
  volume={},
  number={},
  pages={1-6},
  doi={10.1109/ICITSI.2015.7437678}}
"""

_DESCRIPTION = """\
This data contains 3616 lines of Sundanese sentences taken from the online Sundanese language magazine Mangle, West Java Dakwah Council, and Balebat, and translated into Indonesian by several students of the Sundanese language study program UPI Bandung.
"""

_HOMEPAGE = "https://dataverse.telkomuniversity.ac.id/dataset.xhtml?persistentId=doi:10.34820/FK2/HDYWXW"

_LICENSE = "Creative Commons CC0 - No Rights Reserved"

_URLs = {"ind": "https://dataverse.telkomuniversity.ac.id/api/access/datafile/:persistentId?persistentId=doi:10.34820/FK2/HDYWXW/032QZD",
         "sun": "https://dataverse.telkomuniversity.ac.id/api/access/datafile/:persistentId?persistentId=doi:10.34820/FK2/HDYWXW/IVP3G5"}

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class ParallelSuId(datasets.GeneratorBasedBuilder):
    """Parallel Su-Id is a machine translation dataset containing Indonesian-Sundanese parallel sentences collected from the online Sundanese language magazine Mangle, West Java Dakwah Council, and Balebat."""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="parallel_su_id_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="Parallel Su-Id source schema",
            schema="source",
            subset_id="parallel_su_id",
        ),
        SEACrowdConfig(
            name="parallel_su_id_seacrowd_t2t",
            version=datasets.Version(_SEACROWD_VERSION),
            description="Parallel Su-Id Nusantara schema",
            schema="seacrowd_t2t",
            subset_id="parallel_su_id",
        ),
    ]

    DEFAULT_CONFIG_NAME = "parallel_su_id_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features({"id": datasets.Value("string"), "text": datasets.Value("string"), "label": datasets.Value("string")})
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        ind_path = Path(dl_manager.download_and_extract(_URLs["ind"]))
        sun_path = Path(dl_manager.download_and_extract(_URLs["sun"]))
        data_files = {
            "ind": ind_path,
            "sun": sun_path,
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath_dict": data_files},
            )
        ]

    def _generate_examples(self, filepath_dict):
        data = {}
        for lang, path in filepath_dict.items():
            file = open(path, "r")
            data[lang] = []
            for line in file:
                data[lang].append(line)
        if self.config.schema == "source":
            for i in range(len(data[lang])):
                ex = {
                      "id": i, 
                      "text": data['sun'][i].replace("\n",""),
                      "label": data['ind'][i].replace("\n","")
                }
                yield i, ex
        elif self.config.schema == "seacrowd_t2t":
            for i in range(len(data[lang])):
                ex = {
                    "id": i,
                    "text_1": data['sun'][i].replace("\n",""),
                    "text_2": data['ind'][i].replace("\n",""),
                    "text_1_name": "sun",
                    "text_2_name": "ind",
                }
                yield i, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



# Some code referenced from https://huggingface.co/datasets/xnli/blob/main/xnli.py

"""
The Cross-lingual Natural Language Inference (XNLI) corpus is a crowd-sourced collection of 5,000 test and 2,500 dev pairs for the MultiNLI corpus. The pairs are annotated with textual entailment and translated into 14 languages: French, Spanish, German, Greek, Bulgarian, Russian, Turkish, Arabic, Vietnamese, Thai, Chinese, Hindi, Swahili and Urdu. This results in 112.5k annotated pairs. Each premise can be associated with the corresponding hypothesis in the 15 languages, summing up to more than 1.5M combinations.
"""

import csv
import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@InProceedings{conneau2018xnli,
  author = "Conneau, Alexis
        and Rinott, Ruty
        and Lample, Guillaume
        and Williams, Adina
        and Bowman, Samuel R.
        and Schwenk, Holger
        and Stoyanov, Veselin",
  title = "XNLI: Evaluating Cross-lingual Sentence Representations",
  booktitle = "Proceedings of the 2018 Conference on Empirical Methods
               in Natural Language Processing",
  year = "2018",
  publisher = "Association for Computational Linguistics",
  location = "Brussels, Belgium",
}
"""

_DATASETNAME = "xnli"

_DESCRIPTION = """\
XNLI is a subset of a few thousand examples from MNLI which has been translated into a 14 different languages (some low-ish resource). As with MNLI, the goal is to predict textual entailment (does sentence A imply/contradict/neither sentence B) and is a classification task (given two sentences, predict one of three labels).
"""

_HOMEPAGE = "https://github.com/facebookresearch/XNLI"

# We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LANGUAGES = ["tha", "vie"]
_LANGUAGE_MAPPER = {"tha": "th", "vie": "vi"}

_LICENSE = Licenses.CC_BY_NC_4_0.value

_LOCAL = False

_URLS = {
    _DATASETNAME: {
        "train": "https://dl.fbaipublicfiles.com/XNLI/XNLI-MT-1.0.zip",
        "test": "https://dl.fbaipublicfiles.com/XNLI/XNLI-1.0.zip",
    }
}

_SUPPORTED_TASKS = [Tasks.TEXTUAL_ENTAILMENT]

_SOURCE_VERSION = "1.1.0"

_SEACROWD_VERSION = "1.0.0"


class XNLIDataset(datasets.GeneratorBasedBuilder):
    """
    XNLI is an evaluation corpus for language transfer and cross-lingual sentence classification in 15 languages.
    In SeaCrowd, we currently only have Thailand and Vietnam Language.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    subsets = ["xnli.tha", "xnli.vie"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{sub}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{sub} source schema",
            schema="source",
            subset_id=f"{sub}",
        )
        for sub in subsets
    ] + [
        SEACrowdConfig(
            name=f"{sub}_seacrowd_pairs",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{sub} SEACrowd schema",
            schema="seacrowd_pairs",
            subset_id=f"{sub}",
        )
        for sub in subsets
    ]

    DEFAULT_CONFIG_NAME = "xnli.vie_source"
    labels = ["contradiction", "entailment", "neutral"]

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "premise": datasets.Value("string"),
                    "hypothesis": datasets.Value("string"),
                    "label": datasets.ClassLabel(names=self.labels),
                }
            )

        elif self.config.schema == "seacrowd_pairs":
            features = schemas.pairs_features(self.labels)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        xnli_train = os.path.join(data_dir["train"], "XNLI-MT-1.0", "multinli")
        train_data_path = os.path.join(xnli_train, "multinli.train.{}.tsv")

        xnli_test = os.path.join(data_dir["test"], "XNLI-1.0")
        val_data_path = os.path.join(xnli_test, "xnli.dev.tsv")
        test_data_path = os.path.join(xnli_test, "xnli.test.tsv")

        lang = self.config.name.split("_")[0].split(".")[-1]

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_data_path,
                    "split": "train",
                    "language": _LANGUAGE_MAPPER[lang],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": val_data_path,
                    "split": "dev",
                    "language": _LANGUAGE_MAPPER[lang],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": test_data_path,
                    "split": "test",
                    "language": _LANGUAGE_MAPPER[lang],
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str, language: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        if self.config.schema == "source":
            if split == "train":
                file = open(filepath.format(language), encoding="utf-8")
                reader = csv.DictReader(file, delimiter="\t", quoting=csv.QUOTE_NONE)
                for row_idx, row in enumerate(reader):
                    key = str(row_idx)
                    yield key, {
                        "premise": row["premise"],
                        "hypothesis": row["hypo"],
                        "label": row["label"].replace("contradictory", "contradiction"),
                    }
            else:
                with open(filepath, encoding="utf-8") as f:
                    reader = csv.DictReader(f, delimiter="\t", quoting=csv.QUOTE_NONE)
                    for row in reader:
                        if row["language"] == language:
                            yield row["pairID"], {
                                "premise": row["sentence1"],
                                "hypothesis": row["sentence2"],
                                "label": row["gold_label"],
                            }
        elif self.config.schema == "seacrowd_pairs":
            if split == "train":
                file = open(filepath.format(language), encoding="utf-8")
                reader = csv.DictReader(file, delimiter="\t", quoting=csv.QUOTE_NONE)
                for row_idx, row in enumerate(reader):
                    yield str(row_idx), {
                        "id": str(row_idx),
                        "text_1": row["premise"],
                        "text_2": row["hypo"],
                        "label": row["label"].replace("contradictory", "contradiction"),
                    }
            else:
                with open(filepath, encoding="utf-8") as f:
                    reader = csv.DictReader(f, delimiter="\t", quoting=csv.QUOTE_NONE)
                    skip = set()
                    for row in reader:
                        if row["language"] == language:
                            if row["pairID"] in skip:
                                continue
                            skip.add(row["pairID"])
                            yield row["pairID"], {
                                "id": row["pairID"],
                                "text_1": row["sentence1"],
                                "text_2": row["sentence2"],
                                "label": row["gold_label"],
                            }
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



import json
from pathlib import Path
from typing import List

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@INPROCEEDINGS{8629144,  
  author={R. {Jannati} and R. {Mahendra} and C. W. {Wardhana} and M. {Adriani}},
  booktitle={2018 International Conference on Asian Language Processing (IALP)},
  title={Stance Classification Towards Political Figures on Blog Writing},
  year={2018},
  volume={},
  number={},
  pages={96-101},
}
"""

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "id_stance"
_DESCRIPTION = """\
Stance Classification Towards Political Figures on Blog Writing.
This dataset contains dataset from the second research, which is combined from the first research and new dataset.
The dataset consist of 337 data, about five target and every target have 1 different event.
Two label are used: 'For' and 'Againts'.
1. For - the text that is created by author is support the target in an event
2. Against - the text that is created by author is oppose the target in an event
"""
_HOMEPAGE = "https://github.com/reneje/id_stance_dataset_article-Stance-Classification-Towards-Political-Figures-on-Blog-Writing"
_LICENSE = "Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License"
_URLs = {
    _DATASETNAME: "https://raw.githubusercontent.com/reneje/id_stance_dataset_article-Stance-Classification-Towards-Political-Figures-on-Blog-Writing/master/dataset_stance_2_label_2018_building_by_rini.csv"
}
_SUPPORTED_TASKS = [Tasks.TEXTUAL_ENTAILMENT]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


def parse_list(content):
    if (not content):
        return []
    try:
        return json.loads(content)
    except:
        return json.loads("[\"" + content[1:-1].replace("\"", "\\\"") + "\"]")


class IdStance(datasets.GeneratorBasedBuilder):
    """The ID Stance dataset is annotated with a label whether the article is in favor of the person in the context of the event"""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="id_stance_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="IdStance source schema",
            schema="source",
            subset_id="id_stance",
        ),
        SEACrowdConfig(
            name="id_stance_seacrowd_pairs",
            version=datasets.Version(_SEACROWD_VERSION),
            description="IdStance Nusantara schema",
            schema="seacrowd_pairs",
            subset_id="id_stance",
        ),
    ]

    DEFAULT_CONFIG_NAME = "id_stance_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "person": datasets.Value("string"),
                    "event": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "content": datasets.Value("string"),
                    "stance_final": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_pairs":
            features = schemas.pairs_features(["for", "against", "no"])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        data_path = Path(dl_manager.download_and_extract(_URLs[_DATASETNAME]))
        data_files = {
            "train": data_path,
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_files["train"]},
            ),
        ]

    def _generate_examples(self, filepath: Path):
        df = pd.read_csv(filepath, sep=";", header="infer", keep_default_na=False).reset_index()
        df.columns = ["id", "person", "event", "title", "content", "stance_final", ""]
        df.content = df.content.apply(parse_list)

        if self.config.schema == "source":
            for row in df.itertuples():
                ex = {
                    "person": row.person,
                    "event": row.event,
                    "title": row.title,
                    "content": " ".join(row.content),
                    "stance_final": row.stance_final
                }
                yield row.id, ex
        elif self.config.schema == "seacrowd_pairs":
            for row in df.itertuples():
                ex = {
                    "id": row.id,
                    "text_1": row.person + " | " + row.event,
                    "text_2": " ".join([row.title] + row.content),
                    "label": 'against' if row.stance_final == 'againts' else row.stance_final
                }
                yield row.id, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
NTREX-128, a data set for machine translation (MT) evaluation, includes 123 documents \
(1,997 sentences, 42k words) translated from English into 128 target languages. \
9 languages are natively spoken in Southeast Asia, i.e., Burmese, Filipino, \
Hmong, Indonesian, Khmer, Lao, Malay, Thai, and Vietnamese.
"""
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{federmann-etal-2022-ntrex,
    title = "{NTREX}-128 {--} News Test References for {MT} Evaluation of 128 Languages",
    author = "Federmann, Christian  and
      Kocmi, Tom  and
      Xin, Ying",
    editor = "Ahuja, Kabir  and
      Anastasopoulos, Antonios  and
      Patra, Barun  and
      Neubig, Graham  and
      Choudhury, Monojit  and
      Dandapat, Sandipan  and
      Sitaram, Sunayana  and
      Chaudhary, Vishrav",
    booktitle = "Proceedings of the First Workshop on Scaling Up Multilingual Evaluation",
    month = nov,
    year = "2022",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2022.sumeval-1.4",
    pages = "21--24",
}
"""

_DATASETNAME = "ntrex_128"

_DESCRIPTION = """\
NTREX-128, a data set for machine translation (MT) evaluation, includes 123 documents \
(1,997 sentences, 42k words) translated from English into 128 target languages. \
9 languages are natively spoken in Southeast Asia, i.e., Burmese, Filipino, \
Hmong, Indonesian, Khmer, Lao, Malay, Thai, and Vietnamese.
"""

_HOMEPAGE = "https://github.com/MicrosoftTranslator/NTREX"

_LANGUAGES = ["mya", "fil", "ind", "khm", "lao", "zlm", "tha", "vie", "hmv"]

_LICENSE = Licenses.CC_BY_SA_4_0.value

_LOCAL = False

# _MAPPING = {"mya": "mya", "fil": "fil", "ind": "ind", "khm": "khm", "lao": "lao", "zlm": "msa", "tha": "tha", "vie": "vie", "hmv": "hmn"}
_MAPPING = {
    "afr": "afr",
    "amh": "amh",
    "arb": "arb",
    "aze-Latn": "aze-Latn",
    "bak": "bak",
    "bel": "bel",
    "bem": "bem",
    "ben": "ben",
    "bod": "bod",
    "bos": "bos",
    "bul": "bul",
    "cat": "cat",
    "ces": "ces",
    "ckb-Arab": "ckb-Arab",
    "cym": "cym",
    "dan": "dan",
    "deu": "deu",
    "div": "div",
    "dzo": "dzo",
    "ell": "ell",
    "eng-GB": "eng-GB",
    "eng-IN": "eng-IN",
    "eng-US": "eng-US",
    "est": "est",
    "eus": "eus",
    "ewe": "ewe",
    "fao": "fao",
    "fas": "fas",
    "fij": "fij",
    "fil": "fil",
    "fin": "fin",
    "fra": "fra",
    "fra-CA": "fra-CA",
    "fuc": "fuc",
    "gle": "gle",
    "glg": "glg",
    "guj": "guj",
    "hau": "hau",
    "heb": "heb",
    "hin": "hin",
    "hmv": "hmn",
    "hrv": "hrv",
    "hun": "hun",
    "hye": "hye",
    "ibo": "ibo",
    "ind": "ind",
    "isl": "isl",
    "ita": "ita",
    "jpn": "jpn",
    "kan": "kan",
    "kat": "kat",
    "kaz": "kaz",
    "khm": "khm",
    "kin": "kin",
    "kir": "kir",
    "kmr": "kmr",
    "kor": "kor",
    "lao": "lao",
    "lav": "lav",
    "lit": "lit",
    "ltz": "ltz",
    "mal": "mal",
    "mar": "mar",
    "mey": "mey",
    "mkd": "mkd",
    "mlg": "mlg",
    "mlt": "mlt",
    "mon": "mon",
    "mri": "mri",
    "zlm": "msa",
    "mya": "mya",
    "nde": "nde",
    "nep": "nep",
    "nld": "nld",
    "nno": "nno",
    "nob": "nob",
    "nso": "nso",
    "nya": "nya",
    "orm": "orm",
    "pan": "pan",
    "pol": "pol",
    "por": "por",
    "por-BR": "por-BR",
    "prs": "prs",
    "pus": "pus",
    "ron": "ron",
    "rus": "rus",
    "shi": "shi",
    "sin": "sin",
    "slk": "slk",
    "slv": "slv",
    "smo": "smo",
    "sna-Latn": "sna-Latn",
    "snd-Arab": "snd-Arab",
    "som": "som",
    "spa": "spa",
    "spa-MX": "spa-MX",
    "sqi": "sqi",
    "srp-Cyrl": "srp-Cyrl",
    "srp-Latn": "srp-Latn",
    "ssw": "ssw",
    "swa": "swa",
    "swe": "swe",
    "tah": "tah",
    "tam": "tam",
    "tat": "tat",
    "tel": "tel",
    "tgk-Cyrl": "tgk-Cyrl",
    "tha": "tha",
    "tir": "tir",
    "ton": "ton",
    "tsn": "tsn",
    "tuk": "tuk",
    "tur": "tur",
    "uig": "uig",
    "ukr": "ukr",
    "urd": "urd",
    "uzb": "uzb",
    "ven": "ven",
    "vie": "vie",
    "wol": "wol",
    "xho": "xho",
    "yor": "yor",
    "yue": "yue",
    "zho-CN": "zho-CN",
    "zho-TW": "zho-TW",
    "zul": "zul",
}
_URLS = {
    _DATASETNAME: "https://raw.githubusercontent.com/MicrosoftTranslator/NTREX/main/NTREX-128/newstest2019-ref.{lang}.txt",
}

_ALL_LANG = [
    "afr",
    "amh",
    "arb",
    "aze-Latn",
    "bak",
    "bel",
    "bem",
    "ben",
    "bod",
    "bos",
    "bul",
    "cat",
    "ces",
    "ckb-Arab",
    "cym",
    "dan",
    "deu",
    "div",
    "dzo",
    "ell",
    "eng-GB",
    "eng-IN",
    "eng-US",
    "est",
    "eus",
    "ewe",
    "fao",
    "fas",
    "fij",
    "fil",
    "fin",
    "fra",
    "fra-CA",
    "fuc",
    "gle",
    "glg",
    "guj",
    "hau",
    "heb",
    "hin",
    "hmv",
    "hrv",
    "hun",
    "hye",
    "ibo",
    "ind",
    "isl",
    "ita",
    "jpn",
    "kan",
    "kat",
    "kaz",
    "khm",
    "kin",
    "kir",
    "kmr",
    "kor",
    "lao",
    "lav",
    "lit",
    "ltz",
    "mal",
    "mar",
    "mey",
    "mkd",
    "mlg",
    "mlt",
    "mon",
    "mri",
    "zlm",
    "mya",
    "nde",
    "nep",
    "nld",
    "nno",
    "nob",
    "nso",
    "nya",
    "orm",
    "pan",
    "pol",
    "por",
    "por-BR",
    "prs",
    "pus",
    "ron",
    "rus",
    "shi",
    "sin",
    "slk",
    "slv",
    "smo",
    "sna-Latn",
    "snd-Arab",
    "som",
    "spa",
    "spa-MX",
    "sqi",
    "srp-Cyrl",
    "srp-Latn",
    "ssw",
    "swa",
    "swe",
    "tah",
    "tam",
    "tat",
    "tel",
    "tgk-Cyrl",
    "tha",
    "tir",
    "ton",
    "tsn",
    "tuk",
    "tur",
    "uig",
    "ukr",
    "urd",
    "uzb",
    "ven",
    "vie",
    "wol",
    "xho",
    "yor",
    "yue",
    "zho-CN",
    "zho-TW",
    "zul",
]

# aze-Latn: Azerbaijani (Latin)
# ckb-Arab: Central Kurdish (Sorani)
# eng-GB: English (British), eng-IN: English (India), eng-US: English (US)
# fra: French, fra-CA: French (Canada)
# mya: Myanmar
# por: Portuguese, por-BR: Portuguese (Brazil)
# shi: Shilha
# sna-Latn: Shona (Latin)
# snd-Arab: Sindhi (Arabic)
# spa: Spanish, spa-MX: Spanish (Mexico)
# srp-Cyrl: Serbian (Cyrillic), srp-Latn: Serbian (Latin)
# tgk-Cyrl: Tajik (Cyrillic)
# yue: Cantonese
# zho-CN: Chinese (Simplified), zho-TW: Chinese (Traditional)

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "11.24.2022"

_SEACROWD_VERSION = "1.0.0"


class Ntrex128Dataset(datasets.GeneratorBasedBuilder):
    """NTREX-128, a data set for machine translation (MT) evaluation, includes 123 documents \
    (1,997 sentences, 42k words) translated from English into 128 target languages. \
    9 languages are natively spoken in Southeast Asia, i.e., Burmese, Filipino, \
    Hmong, Indonesian, Khmer, Lao, Malay, Thai, and Vietnamese."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{subset1}_{subset2}_source",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} {subset1}2{subset2} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}_{subset1}_{subset2}",
        )
        for subset2 in _ALL_LANG
        for subset1 in _ALL_LANG
        if subset1 != subset2 and (subset1 in _LANGUAGES or subset2 in _LANGUAGES)
    ] + [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{subset1}_{subset2}_seacrowd_t2t",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} {subset1}2{subset2} SEACrowd schema",
            schema="seacrowd_t2t",
            subset_id=f"{_DATASETNAME}_{subset1}_{subset2}",
        )
        for subset2 in _ALL_LANG
        for subset1 in _ALL_LANG
        if subset1 != subset2 and (subset1 in _LANGUAGES or subset2 in _LANGUAGES)
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_mya_fil_source"

    def _info(self):
        # The format of the source is just texts in different .txt files (each file corresponds to one language).
        # Decided make source schema the same as the seacrowd_t2t schema.
        if self.config.schema == "source" or self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        lang1 = self.config.name.split("_")[2]
        lang2 = self.config.name.split("_")[3]
        lang1_txt_path = Path(dl_manager.download_and_extract(_URLS[_DATASETNAME].format(lang=_MAPPING[lang1])))
        lang2_txt_path = Path(dl_manager.download_and_extract(_URLS[_DATASETNAME].format(lang=_MAPPING[lang2])))
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": [lang1_txt_path, lang2_txt_path]},
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        lang1 = self.config.name.split("_")[2]
        lang2 = self.config.name.split("_")[3]

        texts1 = []
        texts2 = []
        texts1 = open(filepath[0], "r").readlines()
        texts2 = open(filepath[1], "r").readlines()

        if self.config.schema == "source" or self.config.schema == "seacrowd_t2t":
            idx = 0
            for line1, line2 in zip(texts1, texts2):
                ex = {
                    "id": str(idx),
                    "text_1": line1,
                    "text_2": line2,
                    "text_1_name": lang1,
                    "text_2_name": lang2,
                }
                yield idx, ex
                idx += 1
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
from seacrowd.utils import schemas
from seacrowd.utils.common_parser import load_conll_data

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{hoesen2018investigating,
  title={Investigating Bi-LSTM and CRF with POS Tag Embedding for Indonesian Named Entity Tagger},
  author={Devin Hoesen and Ayu Purwarianti},
  booktitle={Proceedings of the 2018 International Conference on Asian Language Processing (IALP)},
  pages={35--38},
  year={2018},
  organization={IEEE}
}

@inproceedings{wilie2020indonlu,
  title={IndoNLU: Benchmark and Resources for Evaluating Indonesian Natural Language Understanding},
  author={Bryan Wilie and Karissa Vincentio and Genta Indra Winata and Samuel Cahyawijaya and X. Li and Zhi Yuan Lim and S. Soleman and R. Mahendra and Pascale Fung and Syafri Bahar and A. Purwarianti},
  booktitle={Proceedings of the 1st Conference of the Asia-Pacific Chapter of the Association for Computational Linguistics and the 10th International Joint Conference on Natural Language Processing},
  year={2020}
}
"""

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "posp"

_DESCRIPTION = """\
POSP is a POS Tagging dataset containing 8400 sentences, collected from Indonesian news website with 26 POS tag classes.
The POS tag labels follow the Indonesian Association of Computational Linguistics (INACL) POS Tagging Convention.
POSP dataset is splitted into 3 sets with 6720 train, 840 validation, and 840 test data.
"""

_HOMEPAGE = "https://github.com/IndoNLP/indonlu"

_LICENSE = "Creative Common Attribution Share-Alike 4.0 International"

_URLS = {
    _DATASETNAME: {
        "train": "https://raw.githubusercontent.com/IndoNLP/indonlu/master/dataset/posp_pos-prosa/train_preprocess.txt",
        "validation": "https://raw.githubusercontent.com/IndoNLP/indonlu/master/dataset/posp_pos-prosa/valid_preprocess.txt",
        "test": "https://raw.githubusercontent.com/IndoNLP/indonlu/master/dataset/posp_pos-prosa/test_preprocess.txt",
    }
}

_SUPPORTED_TASKS = [Tasks.POS_TAGGING]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class POSPDataset(datasets.GeneratorBasedBuilder):
    """POSP is a POS Tagging dataset containing 8400 sentences, collected from Indonesian news website with 26 POS tag classes."""

    label_classes = [
        "B-PPO",
        "B-KUA",
        "B-ADV",
        "B-PRN",
        "B-VBI",
        "B-PAR",
        "B-VBP",
        "B-NNP",
        "B-UNS",
        "B-VBT",
        "B-VBL",
        "B-NNO",
        "B-ADJ",
        "B-PRR",
        "B-PRK",
        "B-CCN",
        "B-$$$",
        "B-ADK",
        "B-ART",
        "B-CSN",
        "B-NUM",
        "B-SYM",
        "B-INT",
        "B-NEG",
        "B-PRI",
        "B-VBE",
    ]

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="posp_source",
            version=SOURCE_VERSION,
            description="POSP source schema",
            schema="source",
            subset_id="posp",
        ),
        SEACrowdConfig(
            name="posp_seacrowd_seq_label",
            version=SEACROWD_VERSION,
            description="POSP Nusantara schema",
            schema="seacrowd_seq_label",
            subset_id="posp",
        ),
    ]

    DEFAULT_CONFIG_NAME = "posp_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "index": datasets.Value("string"),
                    "tokens": [datasets.Value("string")],
                    "pos_tags": [datasets.Value("string")],
                }
            )
        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(self.label_classes)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_dir["test"],
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_dir["validation"],
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        conll_dataset = load_conll_data(filepath)

        if self.config.schema == "source":
            for i, row in enumerate(conll_dataset):
                ex = {
                    "index": str(i),
                    "tokens": row["sentence"],
                    "pos_tags": row["label"],
                }
                yield i, ex

        elif self.config.schema == "seacrowd_seq_label":
            for i, row in enumerate(conll_dataset):
                ex = {
                    "id": str(i),
                    "tokens": row["sentence"],
                    "labels": row["label"],
                }
                yield i, ex



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" \
The dataset contribution of this study is a compilation of short fictional stories \
written in Bikol for readability assessment. The data was combined other collected \
Philippine language corpora, such as Tagalog and Cebuano. The data from these languages \
are all distributed across the Philippine elementary system's first three grade \
levels (L1, L2, L3). We sourced this dataset from Let's Read Asia (LRA), Bloom Library, \
Department of Education, and Adarna House.
"""

from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{imperial-kochmar-2023-automatic,
    title = "Automatic Readability Assessment for Closely Related Languages",
    author = "Imperial, Joseph Marvin  and
      Kochmar, Ekaterina",
    editor = "Rogers, Anna  and
      Boyd-Graber, Jordan  and
      Okazaki, Naoaki",
    booktitle = "Findings of the Association for Computational Linguistics: ACL 2023",
    month = jul,
    year = "2023",
    address = "Toronto, Canada",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2023.findings-acl.331",
    doi = "10.18653/v1/2023.findings-acl.331",
    pages = "5371--5386",
    abstract = "In recent years, the main focus of research on automatic readability assessment (ARA) \
    has shifted towards using expensive deep learning-based methods with the primary goal of increasing models{'} accuracy. \
    This, however, is rarely applicable for low-resource languages where traditional handcrafted features are still \
    widely used due to the lack of existing NLP tools to extract deeper linguistic representations. In this work, \
    we take a step back from the technical component and focus on how linguistic aspects such as mutual intelligibility \
    or degree of language relatedness can improve ARA in a low-resource setting. We collect short stories written in three \
    languages in the Philippines{---}Tagalog, Bikol, and Cebuano{---}to train readability assessment models and explore the \
    interaction of data and features in various cross-lingual setups. Our results show that the inclusion of CrossNGO, \
    a novel specialized feature exploiting n-gram overlap applied to languages with high mutual intelligibility, \
    significantly improves the performance of ARA models compared to the use of off-the-shelf large multilingual \
    language models alone. Consequently, when both linguistic representations are combined, we achieve state-of-the-art \
    results for Tagalog and Cebuano, and baseline scores for ARA in Bikol.",
}
"""

_DATASETNAME = "ara_close"

_DESCRIPTION = """\
The dataset contribution of this study is a compilation of short fictional stories \
written in Bikol for readability assessment. The data was combined other collected \
Philippine language corpora, such as Tagalog and Cebuano. The data from these languages \
are all distributed across the Philippine elementary system's first three grade \
levels (L1, L2, L3). We sourced this dataset from Let's Read Asia (LRA), Bloom Library, \
Department of Education, and Adarna House. \
"""

_HOMEPAGE = "https://github.com/imperialite/ara-close-lang"

_LANGUAGES = ["bcl", "ceb"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_LICENSE = Licenses.CC_BY_4_0.value  # example: Licenses.MIT.value, Licenses.CC_BY_NC_SA_4_0.value, Licenses.UNLICENSE.value, Licenses.UNKNOWN.value

_LOCAL = False

_URLS = {
    "bcl": "https://raw.githubusercontent.com/imperialite/ara-close-lang/main/data/bikol/bik_all_data.txt",
    # 'tgl': '', # file for tgl language was deleted
    "ceb": "https://raw.githubusercontent.com/imperialite/ara-close-lang/main/data/cebuano/ceb_all_data.txt",
}

_SUPPORTED_TASKS = [Tasks.READABILITY_ASSESSMENT]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class AraCloseDataset(datasets.GeneratorBasedBuilder):
    f"""{_DESCRIPTION}"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [SEACrowdConfig(name=f"{_DATASETNAME}_{lang}_source", version=datasets.Version(_SOURCE_VERSION), description=f"{_DATASETNAME} source schema", schema="source", subset_id=f"{_DATASETNAME}",) for lang in _LANGUAGES] + [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{lang}_seacrowd_text",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_text",
            subset_id=f"{_DATASETNAME}",
        )
        for lang in _LANGUAGES
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "title": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "label": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(["1", "2", "3"])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        lang = self.config.name.split("_")[2]
        if lang in _LANGUAGES:
            data_path = Path(dl_manager.download_and_extract(_URLS[lang]))
        else:
            data_path = [Path(dl_manager.download_and_extract(_URLS[lang])) for lang in _LANGUAGES]

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_path,
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        lang = self.config.name.split("_")[2]
        if lang in _LANGUAGES:
            file_content = open(filepath, "r").readlines()
        else:
            file_content = []
            for path in filepath:
                lines = open(path, "r").readlines()
                file_content.extend(lines)

        if self.config.schema == "source":
            idx = 0
            for line in file_content:
                split_data = line.strip().split(",")
                title = split_data[0]
                label = split_data[1]
                text = ",".join(split_data[2:])
                ex = {"title": title, "text": text, "label": label}
                yield idx, ex
                idx += 1

        elif self.config.schema == "seacrowd_text":
            idx = 0
            for line in file_content:
                split_data = line.strip().split(",")
                title = split_data[0]
                label = split_data[1]
                text = ",".join(split_data[2:])
                ex = {
                    "id": idx,
                    "text": text,
                    "label": label,
                }
                yield idx, ex
                idx += 1
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



from pathlib import Path
from typing import List

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import DEFAULT_SEACROWD_VIEW_NAME, DEFAULT_SOURCE_VIEW_NAME, Tasks

_DATASETNAME = "id_hoax_news"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["ind"]  # We follow ISO639-3 langauge code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False
_CITATION = """\
@INPROCEEDINGS{8265649,  author={Pratiwi, Inggrid Yanuar Risca and Asmara, Rosa Andrie and Rahutomo, Faisal},  booktitle={2017 11th International Conference on Information & Communication Technology and System (ICTS)},   title={Study of hoax news detection using naïve bayes classifier in Indonesian language},   year={2017},  volume={},  number={},  pages={73-78},  doi={10.1109/ICTS.2017.8265649}}
"""

_DESCRIPTION = """\
This research proposes to build an automatic hoax news detection and collects 250 pages of hoax and valid news articles in Indonesian language.
Each data sample is annotated by three reviewers and the final taggings are obtained by voting of those three reviewers.
"""

_HOMEPAGE = "https://data.mendeley.com/datasets/p3hfgr5j3m/1"

_LICENSE = "Creative Commons Attribution 4.0 International"

_URLs = {
    "train": "https://data.mendeley.com/public-files/datasets/p3hfgr5j3m/files/38bfcff2-8a32-4920-9c26-4f63b5b2dad8/file_downloaded",
}

_SUPPORTED_TASKS = [Tasks.HOAX_NEWS_CLASSIFICATION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IdHoaxNews(datasets.GeneratorBasedBuilder):

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="id_hoax_news_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="Hoax News source schema",
            schema="source",
            subset_id="id_hoax_news",
        ),
        SEACrowdConfig(
            name="id_hoax_news_seacrowd_text",
            version=datasets.Version(_SEACROWD_VERSION),
            description="Hoax News Nusantara schema",
            schema="seacrowd_text",
            subset_id="id_hoax_news",
        ),
    ]

    DEFAULT_CONFIG_NAME = "id_hoax_news_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "index": datasets.Value("string"),
                    "news": datasets.Value("string"),
                    "label": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(["Valid", "Hoax"])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        train_tsv_path = Path(dl_manager.download_and_extract(_URLs["train"]))
        data_files = {
            "train": train_tsv_path / "250 news with valid hoax label.csv",
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_files["train"]},
            ),
        ]

    def _generate_examples(self, filepath: Path):
        news_file = open(filepath, 'r', encoding='ISO-8859-1')
        lines = news_file.readlines()
        news = []
        labels = []

        curr_news = ''
        for l in lines[1:]:
            l = l.replace('\n', '')
            if ';Valid' in l:
                curr_news += l.replace(';Valid', '')
                news.append(curr_news)
                labels.append('Valid')
                curr_news = ''
            elif ';Hoax' in l:
                curr_news += l.replace(';Hoax', '')
                news.append(curr_news)
                labels.append('Hoax')
                curr_news = ''
            else:
                curr_news += l + ' '

        if self.config.schema == "source":
            for i in range(len(news)):
                ex = {"index": str(i), "news": news[i], "label": labels[i]}
                yield i, ex
        elif self.config.schema == "seacrowd_text":
            for i in range(len(news)):
                ex = {"id": str(i), "text": news[i], "label": labels[i]}
                yield i, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{pimentel-ryskina-etal-2021-sigmorphon,
    title = "SIGMORPHON 2021 Shared Task on Morphological Reinflection: Generalization Across Languages",
    author = "Pimentel, Tiago  and
      Ryskina, Maria  and
      Mielke, Sabrina J.  and
      Wu, Shijie  and
      Chodroff, Eleanor  and
      Leonard, Brian  and
      Nicolai, Garrett  and
      Ghanggo Ate, Yustinus  and
      Khalifa, Salam  and
      Habash, Nizar  and
      El-Khaissi, Charbel  and
      Goldman, Omer  and
      Gasser, Michael  and
      Lane, William  and
      Coler, Matt  and
      Oncevay, Arturo  and
      Montoya Samame, Jaime Rafael  and
      Silva Villegas, Gema Celeste  and
      Ek, Adam  and
      Bernardy, Jean-Philippe  and
      Shcherbakov, Andrey  and
      Bayyr-ool, Aziyana  and
      Sheifer, Karina  and
      Ganieva, Sofya  and
      Plugaryov, Matvey  and
      Klyachko, Elena  and
      Salehi, Ali  and
      Krizhanovsky, Andrew  and
      Krizhanovsky, Natalia  and
      Vania, Clara  and
      Ivanova, Sardana  and
      Salchak, Aelita  and
      Straughn, Christopher  and
      Liu, Zoey  and
      Washington, Jonathan North  and
      Ataman, Duygu  and
      Kiera{\'s}, Witold  and
      Woli{\'n}ski, Marcin  and
      Suhardijanto, Totok  and
      Stoehr, Niklas  and
      Nuriah, Zahroh  and
      Ratan, Shyam  and
      Tyers, Francis M.  and
      Ponti, Edoardo M.  and
      Aiton, Grant  and
      Hatcher, Richard J.  and
      Prud'hommeaux, Emily  and
      Kumar, Ritesh  and
      Hulden, Mans  and
      Barta, Botond  and
      Lakatos, Dorina  and
      Szolnok, G{\'a}bor  and
      {\'A}cs, Judit  and
      Raj, Mohit  and
      Yarowsky, David  and
      Cotterell, Ryan  and
      Ambridge, Ben  and
      Vylomova, Ekaterina",
    booktitle = "Proceedings of the 18th SIGMORPHON Workshop on Computational Research in Phonetics, Phonology, and Morphology",
    month = aug,
    year = "2021",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2021.sigmorphon-1.25",
    doi = "10.18653/v1/2021.sigmorphon-1.25",
    pages = "229--259"
}"""

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "unimorph_id"

_DESCRIPTION = """\
The UniMorph project, Indonesian chapter.
Due to sparsity of UniMorph original parsing, raw source is used instead.
Original parsing can be found on https://huggingface.co/datasets/universal_morphologies/blob/2.3.2/universal_morphologies.py
"""

_HOMEPAGE = "https://github.com/unimorph/ind"

_LICENSE = "Creative Commons Attribution-ShareAlike 3.0 Unported (CC BY-SA 3.0)"

_URLS = {
    _DATASETNAME: "https://raw.githubusercontent.com/unimorph/ind/main/ind",
}

_SUPPORTED_TASKS = [Tasks.MORPHOLOGICAL_INFLECTION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class UnimorphIdDataset(datasets.GeneratorBasedBuilder):
    """The UniMorph project, Indonesian chapter."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    label_classes = [
        "",
        "1",
        "2",
        "3",
        "ACT",
        "ADJ",
        "ADV",
        "APPL",
        "CAUS",
        "DEF",
        "FEM",
        "FOC",
        "ITER",
        "MASC",
        "N",
        "NEG",
        "NEUT",
        "PASS",
        "POS",
        "PSS1S",
        "PSS2S",
        "PSS3S",
        "SG",
        "SPRL",
        "TR",
        "V",
    ]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_pairs_multi",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} Nusantara schema",
            schema="seacrowd_pairs_multi",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "lemma": datasets.Value("string"),
                    "form": datasets.Value("string"),
                    "tag": [datasets.Value("string")],
                }
            )

        elif self.config.schema == "seacrowd_pairs_multi":
            features = schemas.pairs_multi_features(self.label_classes)

        else:
            raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        data_path = dl_manager.download(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_path},
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        with open(filepath, "r", encoding="utf8") as f:
            dataset = list(map(lambda l: l.rstrip("\r\n").split("\t"), f))

        _assert = set(map(len, dataset))
        if _assert != {3}:
            raise AssertionError(f"Expecting exactly 3 fields (lemma, form, tag/category), but found: {_assert}")

        def _raw2schema(line):
            return {
                "lemma": line[0],
                "form": line[1],
                "tag": line[2].split(";"),
            }

        dataset = list(map(_raw2schema, dataset))

        if self.config.schema == "source":
            for key, example in enumerate(dataset):
                yield key, example

        elif self.config.schema == "seacrowd_pairs_multi":
            for key, ex in enumerate(dataset):
                yield key, {
                    "id": str(key),
                    "text_1": ex["lemma"],
                    "text_2": ex["form"],
                    "label": ex["tag"],
                }

        else:
            raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")



import json
import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = r"""\
@article{lewis2019mlqa,
    author={Lewis, Patrick and O\{g}uz, Barlas and Rinott, Ruty and Riedel, Sebastian and Schwenk, Holger},
    title={MLQA: Evaluating Cross-lingual Extractive Question Answering},
    journal={arXiv preprint arXiv:1910.07475},
    year={2019}
}
"""

_DATASETNAME = "mlqa"

_DESCRIPTION = """\
MLQA (MultiLingual Question Answering) is a benchmark dataset for evaluating cross-lingual question answering performance.
MLQA consists of over 5K extractive QA instances (12K in English) in SQuAD format in seven languages - English, Arabic,
German, Spanish, Hindi, Vietnamese and Simplified Chinese. MLQA is highly parallel, with QA instances parallel between
4 different languages on average.
"""

_HOMEPAGE = "https://github.com/facebookresearch/MLQA"
_LICENSE = Licenses.CC_BY_SA_3_0.value
_LANGUAGES = ["vie"]
_URL = "https://dl.fbaipublicfiles.com/MLQA/"
_DEV_TEST_URL = "MLQA_V1.zip"
_TRANSLATE_TEST_URL = "mlqa-translate-test.tar.gz"
_TRANSLATE_TRAIN_URL = "mlqa-translate-train.tar.gz"
_SUPPORTED_TASKS = [Tasks.QUESTION_ANSWERING]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"

_LOCAL = False


class MLQADataset(datasets.GeneratorBasedBuilder):
    """
    MLQA (MultiLingual Question Answering) is a benchmark dataset for evaluating cross-lingual question answering performance.
    MLQA consists of over 5K extractive QA instances (12K in English) in SQuAD format in seven languages - English, Arabic,
    German, Spanish, Hindi, Vietnamese and Simplified Chinese. MLQA is highly parallel, with QA instances parallel between
    4 different languages on average.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    subsets = [
        "mlqa-translate-test.vi",
        "mlqa-translate-train.vi",
        "mlqa.vi.ar",
        "mlqa.vi.de",
        "mlqa.vi.zh",
        "mlqa.vi.en",
        "mlqa.vi.es",
        "mlqa.vi.hi",
        "mlqa.vi.vi",
        "mlqa.ar.vi",
        "mlqa.de.vi",
        "mlqa.zh.vi",
        "mlqa.en.vi",
        "mlqa.es.vi",
        "mlqa.hi.vi",
    ]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="{sub}_source".format(sub=subset),
            version=datasets.Version(_SOURCE_VERSION),
            description="{sub} source schema".format(sub=subset),
            schema="source",
            subset_id="{sub}".format(sub=subset),
        )
        for subset in subsets
    ] + [
        SEACrowdConfig(
            name="{sub}_seacrowd_qa".format(sub=subset),
            version=datasets.Version(_SEACROWD_VERSION),
            description="{sub} SEACrowd schema".format(sub=subset),
            schema="seacrowd_qa",
            subset_id="{sub}".format(sub=subset),
        )
        for subset in subsets
    ]

    DEFAULT_CONFIG_NAME = "mlqa.vi.vi_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {"context": datasets.Value("string"), "question": datasets.Value("string"), "answers": datasets.Features({"answer_start": [datasets.Value("int64")], "text": [datasets.Value("string")]}), "id": datasets.Value("string")}
            )
        elif self.config.schema == "seacrowd_qa":
            features = schemas.qa_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        name_split = self.config.name.split("_")
        url = ""
        data_path = ""

        if name_split[0].startswith("mlqa-translate-train"):
            config_name, lang = name_split[0].split(".")
            url = _URL + _TRANSLATE_TRAIN_URL
            data_path = dl_manager.download(url)
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    # Whatever you put in gen_kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": f"{config_name}/{lang}_squad-translate-train-train-v1.1.json",
                        "files": dl_manager.iter_archive(data_path),
                        "split": "train",
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    gen_kwargs={
                        "filepath": f"{config_name}/{lang}_squad-translate-train-dev-v1.1.json",
                        "files": dl_manager.iter_archive(data_path),
                        "split": "test",
                    },
                ),
            ]

        elif name_split[0].startswith("mlqa-translate-test"):
            config_name, lang = name_split[0].split(".")
            url = _URL + _TRANSLATE_TEST_URL
            data_path = dl_manager.download(url)
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    gen_kwargs={
                        "filepath": f"{config_name}/translate-test-context-{lang}-question-{lang}.json",
                        "files": dl_manager.iter_archive(data_path),
                        "split": "test",
                    },
                ),
            ]

        elif name_split[0].startswith("mlqa."):
            url = _URL + _DEV_TEST_URL
            data_path = dl_manager.download_and_extract(url)
            ctx_lang, qst_lang = name_split[0].split(".")[1:]
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    gen_kwargs={
                        "filepath": os.path.join(
                            os.path.join(data_path, "MLQA_V1/dev"),
                            f"dev-context-{ctx_lang}-question-{qst_lang}.json",
                        ),
                        "split": "dev",
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    gen_kwargs={
                        "filepath": os.path.join(
                            os.path.join(data_path, "MLQA_V1/test"),
                            f"test-context-{ctx_lang}-question-{qst_lang}.json",
                        ),
                        "split": "test",
                    },
                ),
            ]
        elif name_split[0] == "mlqa":
            url = _URL + _DEV_TEST_URL
            data_path = dl_manager.download_and_extract(url)
            ctx_lang = qst_lang = "vi"
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    gen_kwargs={
                        "filepath": os.path.join(
                            os.path.join(data_path, "MLQA_V1/dev"),
                            f"dev-context-{ctx_lang}-question-{qst_lang}.json",
                        ),
                        "split": "dev",
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    gen_kwargs={
                        "filepath": os.path.join(
                            os.path.join(data_path, "MLQA_V1/test"),
                            f"test-context-{ctx_lang}-question-{qst_lang}.json",
                        ),
                        "split": "test",
                    },
                ),
            ]

    def _generate_examples(self, filepath: Path, split: str, files=None) -> Tuple[int, Dict]:
        is_config_ok = True
        if self.config.name.startswith("mlqa-translate"):
            for path, f in files:
                if path == filepath:
                    data = json.loads(f.read().decode("utf-8"))
                    break

        elif self.config.schema == "source" or self.config.schema == "seacrowd_qa":
            with open(filepath, encoding="utf-8") as f:
                data = json.load(f)
        else:
            is_config_ok = False
            raise ValueError(f"Invalid config: {self.config.name}")

        if is_config_ok:
            count = 0
            for examples in data["data"]:
                for example in examples["paragraphs"]:
                    context = example["context"]
                    for qa in example["qas"]:
                        question = qa["question"]
                        id_ = qa["id"]
                        answers = qa["answers"]
                        answers_start = [answer["answer_start"] for answer in answers]
                        answers_text = [answer["text"] for answer in answers]

                        if self.config.schema == "source":
                            yield count, {
                                "context": context,
                                "question": question,
                                "answers": {"answer_start": answers_start, "text": answers_text},
                                "id": id_,
                            }
                            count += 1

                        elif self.config.schema == "seacrowd_qa":
                            yield count, {"question_id": id_, "context": context, "question": question, "answer": {"answer_start": answers_start[0], "text": answers_text[0]}, "id": id_, "choices": [], "type": "extractive", "document_id": count, "meta":{}}
                            count += 1



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@InProceedings{,
author="Jiang, Shengyi
and Huang, Xiuwen
and Cai, Xiaonan
and Lin, Nankai",
title="Pre-trained Models and Evaluation Data for the Myanmar Language",
booktitle="The 28th International Conference on Neural Information Processing",
year="2021",
publisher="Springer International Publishing",
address="Cham",
}
"""

_DATASETNAME = "gklmip_sentiment"
_DESCRIPTION = """\
The GKLMIP Product Sentiment Dataset is a Burmese dataset for sentiment analysis. \
It was created by crawling comments on an e-commerce website. The sentiment labels range \
from 1 to 5, with 1 and 2 being negative, 3 and 4 being neutral, and 5 being positive.
"""

_HOMEPAGE = "https://github.com/GKLMIP/Pretrained-Models-For-Myanmar/tree/main"
_LANGUAGES = ["mya"]
_LICENSE = Licenses.UNKNOWN.value
_LOCAL = False

_URLS = {
    _DATASETNAME: "https://github.com/GKLMIP/Pretrained-Models-For-Myanmar/raw/main/Product%20Sentiment%20Dataset.zip",
}

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_LABELS = [1, 2, 3, 4, 5]


class GklmipSentimentDataset(datasets.GeneratorBasedBuilder):
    """The GKLMIP Product Sentiment Dataset is a Burmese dataset for sentiment analysis."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    SEACROWD_SCHEMA_NAME = "text"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({"bpe": datasets.Value("string"), "text": datasets.Value("string"), "label": datasets.Value("string")})

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text_features(_LABELS)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "product_sentiment_dataset_train.json"),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "product_sentiment_dataset_test.json"),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "product_sentiment_dataset_dev.json"),
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        with open(filepath) as file:
            dataset = json.load(file)

        if self.config.schema == "source":
            for i, line in enumerate(dataset):
                yield i, {"bpe": line["bpe"], "text": line["text"], "label": line["label"]}

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            for i, line in enumerate(dataset):
                yield i, {"id": i, "text": line["text"], "label": line["label"]}



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
MIRACL is a multilingual dataset for ad hoc retrieval across 18 languages that 
collectively encompass over three billion native speakers around the world. 
This resource is designed to support monolingual retrieval tasks, where the 
queries and the corpora are in the same language. In total, we have gathered 
over 726k high-quality relevance judgments for 78k queries over Wikipedia in 
these languages, where all annotations have been performed by native speakers. 
MIRACL covers Indonesian and Thai languages
"""

from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses

from collections import defaultdict

_CITATION = """\
@article{10.1162/tacl_a_00595,
    author = {Zhang, Xinyu and Thakur, Nandan and Ogundepo, Odunayo and Kamalloo, Ehsan and Alfonso-Hermelo, David and Li, Xiaoguang and Liu, Qun and Rezagholizadeh, Mehdi and Lin, Jimmy},
    title = "{MIRACL: A Multilingual Retrieval Dataset Covering 18 Diverse Languages}",
    journal = {Transactions of the Association for Computational Linguistics},
    volume = {11},
    pages = {1114-1131},
    year = {2023},
    month = {09},
    abstract = "{MIRACL is a multilingual dataset for ad hoc retrieval across 18 languages that collectively encompass over three billion native speakers around the world. This resource is designed to support monolingual retrieval tasks, where the queries and the corpora are in the same language. In total, we have gathered over 726k high-quality relevance judgments for 78k queries over Wikipedia in these languages, where all annotations have been performed by native speakers hired by our team. MIRACL covers languages that are both typologically close as well as distant from 10 language families and 13 sub-families, associated with varying amounts of publicly available resources. Extensive automatic heuristic verification and manual assessments were performed during the annotation process to control data quality. In total, MIRACL represents an investment of around five person-years of human annotator effort. Our goal is to spur research on improving retrieval across a continuum of languages, thus enhancing information access capabilities for diverse populations around the world, particularly those that have traditionally been underserved. MIRACL is available at http://miracl.ai/.}",
    issn = {2307-387X},
    doi = {10.1162/tacl_a_00595},
    url = {https://doi.org/10.1162/tacl\_a\_00595},
    eprint = {https://direct.mit.edu/tacl/article-pdf/doi/10.1162/tacl\_a\_00595/2157340/tacl\_a\_00595.pdf},
}
"""


_DATASETNAME = "miracl"

_DESCRIPTION = """\
MIRACL is a multilingual dataset for ad hoc retrieval across 18 languages that collectively encompass over three billion native speakers around the world. This resource is designed to support monolingual retrieval tasks, where the queries and the corpora are in the same language. In total, we have gathered over 726k high-quality relevance judgments for 78k queries over Wikipedia in these languages, where all annotations have been performed by native speakers. MIRACL covers Indonesian and Thai languages. Before using this dataloader, please accept the acknowledgement at https://huggingface.co/datasets/miracl/miracl and use huggingface-cli login for authentication.
"""

_HOMEPAGE = "https://project-miracl.github.io/"

_LANGUAGES = ["ind", "tha"]

_LICENSE = Licenses.APACHE_2_0.value

_LANGUAGE_MAP = {
    "id": "Thai", 
    "th": "Indonesian"
}

_URLS = {_DATASETNAME: {lang: {} for lang in _LANGUAGE_MAP}}

for lang in _LANGUAGE_MAP:
    _URLS[_DATASETNAME][lang]['train'] = [
        f'https://huggingface.co/datasets/miracl/miracl/resolve/main/miracl-v1.0-{lang}/topics/topics.miracl-v1.0-{lang}-train.tsv',
        f'https://huggingface.co/datasets/miracl/miracl/resolve/main/miracl-v1.0-{lang}/qrels/qrels.miracl-v1.0-{lang}-train.tsv',
    ]

    _URLS[_DATASETNAME][lang]['dev'] = [
        f'https://huggingface.co/datasets/miracl/miracl/resolve/main/miracl-v1.0-{lang}/topics/topics.miracl-v1.0-{lang}-dev.tsv',
        f'https://huggingface.co/datasets/miracl/miracl/resolve/main/miracl-v1.0-{lang}/qrels/qrels.miracl-v1.0-{lang}-dev.tsv',
    ]
    
    _URLS[_DATASETNAME][lang]['testB'] =[
        f'https://huggingface.co/datasets/miracl/miracl/resolve/main/miracl-v1.0-{lang}/topics/topics.miracl-v1.0-{lang}-test-b.tsv',
    ]

    _URLS[_DATASETNAME][lang]['testA'] = [
        f'https://huggingface.co/datasets/miracl/miracl/resolve/main/miracl-v1.0-{lang}/topics/topics.miracl-v1.0-{lang}-test-a.tsv',
    ]


_SUPPORTED_TASKS = [Tasks.TEXT_RETRIEVAL]  # example: [Tasks.TRANSLATION, Tasks.NAMED_ENTITY_RECOGNITION, Tasks.RELATION_EXTRACTION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_LOCAL = False


def load_topic(fn):

    qid2topic = {}
    with open(fn, encoding="utf-8") as f:
        for line in f:
            qid, topic = line.strip().split('\t')
            qid2topic[qid] = topic
    return qid2topic


def load_qrels(fn):
    if fn is None:
        return None

    qrels = defaultdict(dict)
    with open(fn, encoding="utf-8") as f:
        for line in f:
            qid, _, docid, rel = line.strip().split('\t')
            qrels[qid][docid] = int(rel)
    return qrels

def seacrowd_config_constructor(lang, schema, version):
    if lang not in _LANGUAGE_MAP:
        raise ValueError(f"Invalid lang {lang}")

    if schema != "source" and schema != "seacrowd_pairs":
        raise ValueError(f"Invalid schema: {schema}")

    return SEACrowdConfig(
        name="miracl_{lang}_{schema}".format(lang=lang, schema=schema),
        version=datasets.Version(version),
        description="MIRACL {schema} schema for {lang} language".format(lang=_LANGUAGE_MAP[lang], schema=schema),
        schema=schema,
        subset_id="miracl_{lang}".format(lang=lang),
    )

class Miracl(datasets.GeneratorBasedBuilder):
    """MIRACL is a multilingual retrieval dataset that focuses on search across 18 different languages."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [seacrowd_config_constructor(lang, "source", _SOURCE_VERSION) for lang in _LANGUAGE_MAP] + [seacrowd_config_constructor(lang, "seacrowd_pairs", _SEACROWD_VERSION) for lang in _LANGUAGE_MAP]

    DEFAULT_CONFIG_NAME = None

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({
                'query_id': datasets.Value('string'),
                'query': datasets.Value('string'),

                'positive_passages': [{
                    'docid': datasets.Value('string'),
                    'text': datasets.Value('string'), 'title': datasets.Value('string')
                }],
                'negative_passages': [{
                    'docid': datasets.Value('string'),
                    'text': datasets.Value('string'), 'title': datasets.Value('string'),
                }],
            })
        elif self.config.schema == "seacrowd_pairs":
            features = schemas.pairs_features(["pos", "neg", "none"])
            
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        lang = self.config.name.split("_")[1]
        downloaded_files = dl_manager.download_and_extract(urls[lang])

        return [
            datasets.SplitGenerator(
                name="train",
                gen_kwargs={
                    "filepaths": downloaded_files["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name="dev",
                gen_kwargs={
                    "filepaths": downloaded_files["dev"],
                    "split": "dev",
                },
            ),
            datasets.SplitGenerator(
                name="testA",
                gen_kwargs={
                    "filepaths": downloaded_files["testA"],
                    "split": "testA",
                },
            ),
            datasets.SplitGenerator(
                name="testB",
                gen_kwargs={
                    "filepaths": downloaded_files["testB"],
                    "split": "testB",
                },
            )
        ]


    def _generate_examples(self, filepaths: List[str], split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        lang = self.config.name.split("_")[1]

        # the following code except for seacrowd_pairs is taken from the original MIRACL 
        # dataloader implementation
        # https://huggingface.co/datasets/miracl/miracl
        miracl_corpus = datasets.load_dataset('miracl/miracl-corpus', lang)['train']
        docid2doc = {doc['docid']: (doc['title'], doc['text']) for doc in miracl_corpus}
        topic_fn, qrel_fn = (filepaths) if len(filepaths) == 2 else (filepaths[0], None)
        qid2topic = load_topic(topic_fn)
        qrels = load_qrels(qrel_fn)

        if self.config.schema == "source":
            for qid in qid2topic:
                data = {}
                data['query_id'] = qid
                data['query'] = qid2topic[qid]
                
                pos_docids = [docid for docid, rel in qrels[qid].items() if rel == 1] if qrels is not None else []
                neg_docids = [docid for docid, rel in qrels[qid].items() if rel == 0] if qrels is not None else []

                data['positive_passages'] = [{
                    'docid': docid, 
                    **dict(zip(['title', 'text'], docid2doc[docid]))
                } for docid in pos_docids if docid in docid2doc]

                data['negative_passages'] = [{
                    'docid': docid, 
                    **dict(zip(['title', 'text'], docid2doc[docid]))
                } for docid in neg_docids if docid in docid2doc]
                
                yield qid, data

        elif self.config.schema == "seacrowd_pairs":
            id = -1
            for qid in qid2topic:
                pos_docids = [docid for docid, rel in qrels[qid].items() if rel == 1] if qrels is not None else []
                neg_docids = [docid for docid, rel in qrels[qid].items() if rel == 0] if qrels is not None else []

                positive_passages = [{
                    'docid': docid, 
                    **dict(zip(['title', 'text'], docid2doc[docid]))
                } for docid in pos_docids if docid in docid2doc]

                negative_passages = [{
                    'docid': docid, 
                    **dict(zip(['title', 'text'], docid2doc[docid]))
                } for docid in neg_docids if docid in docid2doc]

                # assemble data
                data = {}
                data['text_1'] = qid2topic[qid] # query

                if split in ["testA", "testB"]: # test sets only contains id and query
                    id += 1
                    data['id'] = id
                    data['text_2'] = ""
                    data['label'] = "none"

                    yield id, data
                else:
                    # generate positive pairs
                    for positive_doc in positive_passages:
                        id += 1
                        data['id'] = id
                        # flatten dict contents to String by concatenating title and text separated by double newline
                        data['text_2'] = positive_doc['title'] + "\n\n" + positive_doc["text"]
                        data['label'] = "pos"
                        yield id, data
                    
                    # generate negative pairs
                    for negative_doc in negative_passages:
                        id += 1
                        data['id'] = id
                        # flatten dict contents to String by concatenating title and text separated by double newline
                        data['text_2'] = negative_doc['title'] + "\n\n" + negative_doc["text"]
                        data['label'] = "neg"
                        yield id, data



# coding=utf-8
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{ho2020emotion,
  title={Emotion recognition for vietnamese social media text},
  author={Ho, Vong Anh and Nguyen, Duong Huynh-Cong and Nguyen, Danh Hoang and Pham, Linh Thi-Van and Nguyen, Duc-Vu and Nguyen, Kiet Van and Nguyen, Ngan Luu-Thuy},
  booktitle={Computational Linguistics: 16th International Conference of the Pacific Association for Computational Linguistics, PACLING 2019, Hanoi, Vietnam, October 11--13, 2019, Revised Selected Papers 16},
  pages={319--333},
  year={2020},
  organization={Springer}
}
"""

_DATASETNAME = "uit_vsmec"

_DESCRIPTION = """\
This dataset consists of Vietnamese Facebook comments that were manually annotated for sentiment.
There are seven possible emotion labels: enjoyment, sadness, fear, anger, disgust, surprise or other (for comments with no or neutral emotions).
Two rounds of manual annotations were done to train annotators with tagging and editing guidelines.
Annotation was performed until inter-annotator agreement reached at least 80%.
"""

_HOMEPAGE = "https://drive.google.com/drive/folders/1HooABJyrddVGzll7fgkJ6VzkG_XuWfRu"

_LICENSE = Licenses.UNKNOWN.value

_LANGUAGES = ["vie"]

_LOCAL = False

_URLS = {
    "train": "https://docs.google.com/spreadsheets/export?id=10VYzfK7JLg-vfmqH0UmKX62z_uaXU-Hp&format=csv",
    "valid": "https://docs.google.com/spreadsheets/export?id=1EsSFZ94fj2yTvFKO6EyxM0wBRcG0s1KE&format=csv",
    "test": "https://docs.google.com/spreadsheets/export?id=1D16FCKKgJ0T6t2aSA3biWVwvD9fa4G9a&format=csv",
}

_SUPPORTED_TASKS = [Tasks.EMOTION_CLASSIFICATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class UITVSMECDataset(datasets.GeneratorBasedBuilder):
    """
    This is the main class of SEACrowd dataloader for UIT-VSMEC, focusing on emotion/sentiment classification task.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_text",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_text",
            subset_id=f"{_DATASETNAME}",
        ),
    ]
    LABEL_NAMES = ["Other", "Disgust", "Enjoyment", "Anger", "Surprise", "Sadness", "Fear"]
    DEFAULT_CONFIG_NAME = "uit_vsmec_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({"Emotion": datasets.Value("string"), "Sentence": datasets.Value("string")})

        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(self.LABEL_NAMES)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        path_dict = dl_manager.download_and_extract(_URLS)
        train_path, valid_path, test_path = path_dict["train"], path_dict["valid"], path_dict["test"]

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_path,
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": test_path,
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": valid_path,
                },
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        df = pd.read_csv(filepath).reset_index()
        if self.config.schema == "source":
            for row in df.itertuples():
                ex = {"Emotion": row.Emotion, "Sentence": row.Sentence}
                yield row.index, ex

        elif self.config.schema == "seacrowd_text":
            for row in df.itertuples():
                ex = {"id": str(row.index), "text": row.Sentence, "label": self.LABEL_NAMES.index(row.Emotion)}
                yield row.index, ex



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{myint-oo-etal-2019-neural,
    title = "Neural Machine Translation between {M}yanmar ({B}urmese) and {R}akhine ({A}rakanese)",
    author = "Myint Oo, Thazin and
        Kyaw Thu, Ye and
        Mar Soe, Khin",
    editor = {Zampieri, Marcos and
        Nakov, Preslav and
        Malmasi, Shervin and
        Ljube{\v{s}}i{\'c}, Nikola and
        Tiedemann, J{\"o}rg and
        Ali, Ahmed},
    booktitle = "Proceedings of the Sixth Workshop on {NLP} for Similar Languages, Varieties and Dialects",
    month = jun,
    year = "2019",
    address = "Ann Arbor, Michigan",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/W19-1408",
    doi = "10.18653/v1/W19-1408",
    pages = "80--88",
}
"""

_DATASETNAME = "myanmar_rakhine_parallel"
_DESCRIPTION = """\
The data contains 18,373 Myanmar sentences of the ASEAN-MT Parallel Corpus,
which is a parallel corpus in the travel domain. It contains six main
categories: people (greeting, introduction, and communication), survival
(transportation, accommodation, and finance), food (food, beverages, and
restaurants), fun (recreation, traveling, shopping, and nightlife), resource
(number, time, and accuracy), special needs (emergency and health). Manual
translation into the Rakhine language was done by native Rakhine students from
two Myanmar universities, and the translated corpus was checked by the editor
of a Rakhine newspaper. Word segmentation for Rakhine was done manually, and
there are exactly 123,018 words in total.
"""

_HOMEPAGE = "https://github.com/ye-kyaw-thu/myPar/tree/master/my-rk"
_LANGUAGES = ["mya", "rki"]
_LICENSE = Licenses.GPL_3_0.value
_LOCAL = False
_URLS = {
    "train_mya": "https://raw.githubusercontent.com/ye-kyaw-thu/myPar/master/my-rk/ver-0.1/train.my",
    "dev_mya": "https://raw.githubusercontent.com/ye-kyaw-thu/myPar/master/my-rk/ver-0.1/dev.my",
    "test_mya": "https://raw.githubusercontent.com/ye-kyaw-thu/myPar/master/my-rk/ver-0.1/test.my",
    "train_rki": "https://raw.githubusercontent.com/ye-kyaw-thu/myPar/master/my-rk/ver-0.1/train.rk",
    "dev_rki": "https://raw.githubusercontent.com/ye-kyaw-thu/myPar/master/my-rk/ver-0.1/dev.rk",
    "test_rki": "https://raw.githubusercontent.com/ye-kyaw-thu/myPar/master/my-rk/ver-0.1/test.rk",
}
_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "0.1.0"
_SEACROWD_VERSION = "1.0.0"


class MyanmarRakhineParallel(datasets.GeneratorBasedBuilder):
    """Myanmar-Rakhine Parallel dataset from https://github.com/ye-kyaw-thu/myPar/tree/master/my-rk"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "t2t"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source" or self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text2text_features
        else:
            raise ValueError(f"Invalid config schema: {self.config.schema}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        data_paths = {
            "train_mya": Path(dl_manager.download_and_extract(_URLS["train_mya"])),
            "dev_mya": Path(dl_manager.download_and_extract(_URLS["dev_mya"])),
            "test_mya": Path(dl_manager.download_and_extract(_URLS["test_mya"])),
            "train_rki": Path(dl_manager.download_and_extract(_URLS["train_rki"])),
            "dev_rki": Path(dl_manager.download_and_extract(_URLS["dev_rki"])),
            "test_rki": Path(dl_manager.download_and_extract(_URLS["test_rki"])),
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "mya_filepath": data_paths["train_mya"],
                    "rki_filepath": data_paths["train_rki"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "mya_filepath": data_paths["test_mya"],
                    "rki_filepath": data_paths["test_rki"],
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "mya_filepath": data_paths["dev_mya"],
                    "rki_filepath": data_paths["dev_rki"],
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, mya_filepath: Path, rki_filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        # read mya file
        with open(mya_filepath, "r", encoding="utf-8") as mya_file:
            mya_data = mya_file.readlines()
        mya_data = [s.strip("\n") for s in mya_data]

        # read rki file
        with open(rki_filepath, "r", encoding="utf-8") as rki_file:
            rki_data = rki_file.readlines()
        rki_data = [s.strip("\n") for s in rki_data]

        num_sample = len(mya_data)

        for i in range(num_sample):
            if self.config.schema == "source" or self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                example = {"id": str(i), "text_1": mya_data[i], "text_2": rki_data[i], "text_1_name": "mya", "text_2_name": "rki"}
            yield i, example



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@article{IBROHIM2018222,
title = {A Dataset and Preliminaries Study for Abusive Language Detection in Indonesian Social Media},
journal = {Procedia Computer Science},
volume = {135},
pages = {222-229},
year = {2018},
note = {The 3rd International Conference on Computer Science and Computational Intelligence (ICCSCI 2018) : Empowering Smart Technology in Digital Era for a Better Life},
issn = {1877-0509},
doi = {https://doi.org/10.1016/j.procs.2018.08.169},
url = {https://www.sciencedirect.com/science/article/pii/S1877050918314583},
author = {Muhammad Okky Ibrohim and Indra Budi},
keywords = {abusive language, twitter, machine learning},
abstract = {Abusive language is an expression (both oral or text) that contains abusive/dirty words or phrases both in the context of jokes, a vulgar sex conservation or to cursing someone. Nowadays many people on the internet (netizens) write and post an abusive language in the social media such as Facebook, Line, Twitter, etc. Detecting an abusive language in social media is a difficult problem to resolve because this problem can not be resolved just use word matching. This paper discusses a preliminaries study for abusive language detection in Indonesian social media and the challenge in developing a system for Indonesian abusive language detection, especially in social media. We also built reported an experiment for abusive language detection on Indonesian tweet using machine learning approach with a simple word n-gram and char n-gram features. We use Naive Bayes, Support Vector Machine, and Random Forest Decision Tree classifier to identify the tweet whether the tweet is a not abusive language, abusive but not offensive, or offensive language. The experiment results show that the Naive Bayes classifier with the combination of word unigram + bigrams features gives the best result i.e. 70.06% of F1 - Score. However, if we classifying the tweet into two labels only (not abusive language and abusive language), all classifier that we used gives a higher result (more than 83% of F1 - Score for every classifier). The dataset in this experiment is available for other researchers that interest to improved this study.}
}
"""

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "id_abusive"

_DESCRIPTION = """\
The ID_ABUSIVE dataset is collection of 2,016 informal abusive tweets in Indonesian language,
designed for sentiment analysis NLP task. This dataset is crawled from Twitter, and then filtered
and labelled manually by 20 volunteer annotators. The dataset labelled into three labels namely
not abusive language, abusive but not offensive, and offensive language.
"""

_HOMEPAGE = "https://www.sciencedirect.com/science/article/pii/S1877050918314583"
_LICENSE = "Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International"
_URLS = {
    _DATASETNAME: "https://raw.githubusercontent.com/okkyibrohim/id-abusive-language-detection/master/re_dataset_three_labels.csv",
}
_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IdAbusive(datasets.GeneratorBasedBuilder):
    """The ID_Abusive dataset is collection of Indonesian informal abusive tweets, annotated into three labels namely
    not abusive language, abusive but not offensive, and offensive language."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    LABEL_STRING_MAP = {
        1: "not_abusive",
        2: "abusive",
        3: "abusive_and_offensive",
    }

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="id_abusive_source",
            version=SOURCE_VERSION,
            description="ID Abusive source schema",
            schema="source",
            subset_id="id_abusive",
        ),
        SEACrowdConfig(
            name="id_abusive_seacrowd_text",
            version=SEACROWD_VERSION,
            description="ID Abusive Nusantara schema",
            schema="seacrowd_text",
            subset_id="id_abusive",
        ),
    ]

    DEFAULT_CONFIG_NAME = "id_abusive_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({"tweet": datasets.Value("string"), "label": datasets.Value("string")})
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(["not_abusive", "abusive", "abusive_and_offensive"])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        # Dataset does not have predetermined split, putting all as TRAIN
        urls = _URLS[_DATASETNAME]
        base_dir = Path(dl_manager.download_and_extract(urls))
        data_files = {"train": base_dir}

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_files["train"],
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        # Dataset does not have id, using row index as id
        df = pd.read_csv(filepath).reset_index()
        df.columns = ["id", "label", "tweet"]

        if self.config.schema == "source":
            for row in df.itertuples():
                ex = {
                    "tweet": row.tweet,
                    "label": self.LABEL_STRING_MAP[row.label],
                }
                yield row.id, ex

        elif self.config.schema == "seacrowd_text":
            for row in df.itertuples():
                ex = {
                    "id": str(row.id),
                    "text": row.tweet,
                    "label": self.LABEL_STRING_MAP[row.label],
                }
                yield row.id, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



import csv
import os
from pathlib import Path
from typing import List

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import (DEFAULT_SEACROWD_VIEW_NAME,
                                       DEFAULT_SOURCE_VIEW_NAME, Tasks)

_DATASETNAME = "jv_id_tts"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["jav"]
_LOCAL = False
_CITATION = """\
@inproceedings{sodimana18_sltu,
  author={Keshan Sodimana and Pasindu {De Silva} and Supheakmungkol Sarin and Oddur Kjartansson and Martin Jansche and Knot Pipatsrisawat and Linne Ha},
  title={{A Step-by-Step Process for Building TTS Voices Using Open Source Data and Frameworks for Bangla, Javanese, Khmer, Nepali, Sinhala, and Sundanese}},
  year=2018,
  booktitle={Proc. 6th Workshop on Spoken Language Technologies for Under-Resourced Languages (SLTU 2018)},
  pages={66--70},
  doi={10.21437/SLTU.2018-14}
}
"""

_DESCRIPTION = """\
This data set contains high-quality transcribed audio data for Javanese.
The data set consists of wave files, and a TSV file.
The file line_index.tsv contains a filename and the transcription of audio in the file.
Each filename is prepended with a speaker identification number.
The data set has been manually quality checked, but there might still be errors.
This dataset was collected by Google in collaboration with Gadjah Mada University in Indonesia.
"""

_HOMEPAGE = "http://openslr.org/41/"

_LICENSE = "See https://www.openslr.org/resources/41/LICENSE file for license information. Attribution-ShareAlike 4.0 (CC BY-SA 4.0)."

_URLs = {
    _DATASETNAME: {
        "female": "https://www.openslr.org/resources/41/jv_id_female.zip",
        "male": "https://www.openslr.org/resources/41/jv_id_male.zip",
    }
}

_SUPPORTED_TASKS = [Tasks.TEXT_TO_SPEECH]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class JvIdTTS(datasets.GeneratorBasedBuilder):
    """jv_id_tts contains high-quality Multi-speaker TTS data for Javanese (jv-ID)."""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="jv_id_tts_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="JV_ID_TTS source schema",
            schema="source",
            subset_id="jv_id_tts",
        ),
        SEACrowdConfig(
            name="jv_id_tts_seacrowd_sptext",
            version=datasets.Version(_SEACROWD_VERSION),
            description="JV_ID_TTS Nusantara schema",
            schema="seacrowd_sptext",
            subset_id="jv_id_tts",
        ),
    ]

    DEFAULT_CONFIG_NAME = "jv_id_tts_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "speaker_id": datasets.Value("string"),
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "text": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
            task_templates=[datasets.AutomaticSpeechRecognition(audio_column="audio", transcription_column="text")],
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        male_path = Path(dl_manager.download_and_extract(_URLs[_DATASETNAME]["male"]))
        female_path = Path(dl_manager.download_and_extract(_URLs[_DATASETNAME]["female"]))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "male_filepath": male_path,
                    "female_filepath": female_path,
                },
            ),
        ]

    def _generate_examples(self, male_filepath: Path, female_filepath: Path):

        if self.config.schema == "source" or self.config.schema == "seacrowd_sptext":
            tsv_file = os.path.join(male_filepath, "jv_id_male", "line_index.tsv")
            with open(tsv_file, "r") as file:
                tsv_data = csv.reader(file, delimiter="\t")

                for line in tsv_data:
                    # for male data, the tsv contains three columns
                    audio_id, _, transcription_text = line[0], line[1], line[2]
                    speaker_id = audio_id.split("_")[1]
                    wav_path = os.path.join(male_filepath, "jv_id_male", "wavs", "{}.wav".format(audio_id))

                    if os.path.exists(wav_path):
                        if self.config.schema == "source":
                            ex = {
                                "id": audio_id,
                                "speaker_id": speaker_id,
                                "path": wav_path,
                                "audio": wav_path,
                                "text": transcription_text,
                            }
                            yield audio_id, ex
                        elif self.config.schema == "seacrowd_sptext":
                            ex = {
                                "id": audio_id,
                                "speaker_id": speaker_id,
                                "path": wav_path,
                                "audio": wav_path,
                                "text": transcription_text,
                                "metadata": {
                                    "speaker_age": None,
                                    "speaker_gender": "male",
                                },
                            }
                            yield audio_id, ex

            tsv_file = os.path.join(female_filepath, "jv_id_female", "line_index.tsv")
            with open(tsv_file, "r") as file:
                tsv_data = csv.reader(file, delimiter="\t")

                for line in tsv_data:
                    audio_id, transcription_text = line[0], line[1]
                    speaker_id = audio_id.split("_")[1]
                    wav_path = os.path.join(female_filepath, "jv_id_female", "wavs", "{}.wav".format(audio_id))

                    if os.path.exists(wav_path):
                        if self.config.schema == "source":
                            ex = {
                                "id": audio_id,
                                "speaker_id": speaker_id,
                                "path": wav_path,
                                "audio": wav_path,
                                "text": transcription_text,
                            }
                            yield audio_id, ex
                        elif self.config.schema == "seacrowd_sptext":
                            ex = {
                                "id": audio_id,
                                "speaker_id": speaker_id,
                                "path": wav_path,
                                "audio": wav_path,
                                "text": transcription_text,
                                "metadata": {
                                    "speaker_age": None,
                                    "speaker_gender": "female",
                                },
                            }
                            yield audio_id, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")




import os
from pathlib import Path, PureWindowsPath
from typing import Dict, List, Tuple

import cv2
import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{tupal4476867fsl105,
  title={FSL105: The Video Filipino Sign Language Sign Database of Introductory 105 FSL Signs},
  author={Tupal, Isaiah Jassen Lizaso and Melvin, Cabatuan K},
  journal={Available at SSRN 4476867}
}
"""

_DATASETNAME = "fsl_105"

_DESCRIPTION = """\
FSL-105 is a video dataset for 105 different Filipino Sign Language (FSL) signs.
Each sign is categorized into one of 10 categories and is each represented by approximately 20 four-second video samples.
Signs were performed by adult deaf FSL signers on a blank blue background and reviewed by an FSL expert.
"""

_HOMEPAGE = "https://data.mendeley.com/datasets/48y2y99mb9/2"

_LICENSE = Licenses.CC_BY_4_0.value

_LOCAL = False

_URLS = {
    "clips": "https://prod-dcd-datasets-public-files-eu-west-1.s3.eu-west-1.amazonaws.com/de95a3c3-02f4-4a3f-9a9e-ce2371160275",
    "train": "https://prod-dcd-datasets-public-files-eu-west-1.s3.eu-west-1.amazonaws.com/09c71779-3a2a-4c98-8d9b-0ef74f54d92a",
    "test": "https://prod-dcd-datasets-public-files-eu-west-1.s3.eu-west-1.amazonaws.com/39af8117-6b44-47b9-a551-0bdc40837295",
}

_LANGUAGES = ["psp"]

_SUPPORTED_TASKS = [Tasks.VIDEO_TO_TEXT_RETRIEVAL, Tasks.VIDEO_CAPTIONING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class FSL105Dataset(datasets.GeneratorBasedBuilder):
    """
    FSL-105 is a video dataset for 105 different Filipino Sign Language (FSL) signs.
    Each sign is categorized into one of 10 categories and is each represented by approximately 20 four-second video samples.
    Signs were performed by adult deaf FSL signers on a blank blue background and reviewed by an FSL expert.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_vidtext",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_vidtext",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    category = [
        "CALENDAR",
        "COLOR",
        "DAYS",
        "DRINK",
        "FAMILY",
        "FOOD",
        "GREETING",
        "NUMBER",
        "RELATIONSHIPS",
        "SURVIVAL",
    ]

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "video_path": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "labels": datasets.ClassLabel(names=self.category),
                    "metadata": {
                        "resolution": {
                            "width": datasets.Value("int64"),
                            "height": datasets.Value("int64"),
                        },
                        "duration": datasets.Value("float32"),
                        "fps": datasets.Value("float32"),
                    },
                }
            )

        elif self.config.schema == "seacrowd_vidtext":
            features = schemas.video_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        clips = dl_manager.download_and_extract(_URLS["clips"])
        train = dl_manager.download_and_extract(_URLS["train"])
        test = dl_manager.download_and_extract(_URLS["test"])

        train_df = pd.read_csv(train)
        test_df = pd.read_csv(test)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": {
                        "clips": clips,
                        "data": train_df,
                    },
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": {"clips": clips, "data": test_df},
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        for key, example in filepath["data"].iterrows():
            video = cv2.VideoCapture(os.path.join(filepath["clips"], PureWindowsPath(example["vid_path"]).as_posix()))
            fps = video.get(cv2.CAP_PROP_FPS)
            frame_count = video.get(cv2.CAP_PROP_FRAME_COUNT)
            duration = frame_count / fps
            vid_width = video.get(cv2.CAP_PROP_FRAME_WIDTH)
            vid_height = video.get(cv2.CAP_PROP_FRAME_HEIGHT)

            if self.config.schema == "source":
                yield key, {
                    "id": str(key),
                    "video_path": os.path.join(filepath["clips"], example["vid_path"]),
                    "text": example["label"],
                    "labels": example["category"],
                    "metadata": {
                        "resolution": {
                            "width": vid_width,
                            "height": vid_height,
                        },
                        "duration": duration,
                        "fps": fps,
                    },
                }
            elif self.config.schema == "seacrowd_vidtext":
                yield key, {
                    "id": str(key),
                    "video_path": os.path.join(filepath["clips"], example["vid_path"]),
                    "text": example["label"],
                    "metadata": {
                        "resolution": {
                            "width": vid_width,
                            "height": vid_height,
                        },
                        "duration": duration,
                        "fps": fps,
                    },
                }
